function datatipinfo( val )
%DATATIPINFO Produce a short description of a variable.

%   DATATIPINFO(X) produces a short description of a variable, as for use in
%   debugger DataTips.

%   Copyright 1984-2002 The MathWorks, Inc. 
%   $Revision: 1.13 $  $Date: 2002/04/08 20:51:15 $
%              20041228 CLV Ported to dd-type 

if nargin ~= 1
   return
end

if isempty(val)
   disp('[]');

% The short description amounts to a summary, so we don't want
% to produce a huge amount of information.  As a rough rule of
% thumb, print the full text of the value only if the total
% number of elements is less than an arbitrary cutoff (500).
elseif prod(double(size(val))) <= 500
   eval([inputname(1) '=deal(val);disp(' inputname(1) ')']);

% Otherwise, unless this is a simple string (in which case we
% print the first 500 characters), just print a description of
% the value (along the lines of '40x40 double array')
else
   if isstr(val)
       eval([inputname(1) '=deal(val(1:500));disp(' inputname(1) ')']);
       %disp(val(1:500));
   else
      s = double(size(val));
      switch ndims(val)
         case 2
            fprintf('%dx%d',s(1),s(2));
         case 3
            fprintf('%dx%dx%d',s(1),s(2),s(3)');
         otherwise
            fprintf('%d-D', ndims(val));
      end

      disp([' ' class(val) ' array'])
   end
end
