function [V,D]=eig(A,B,param)
%eig: find eigenvalues and eigenvectors with the power method using the matlab's builtin as a first guess

if ~isa(A,'mp'),A=mp(A);end
%deal with cases where the second parameter might be a mp or not
if (nargin>2) & ~ischar(B)
    %Don't know how the eig(A,B) is defined. Simply provide back the matlab estimate
    switch nargin
        case 2
            [V,D]=builtin('eig',double(A),double(B));
            V=mp(V);D=mp(D);
            return
        case 3
            [V,D]=builtin('eig',double(A),double(B),param);
            V=mp(V);D=mp(D);
            return
        otherwise
            error('Wrong number of parameters in eig')
            return
    end
end
%gather intial estimate values. 
if nargin==2
    [V,D]=builtin('eig',double(A),B);
else
    [V,D]=builtin('eig',double(A));
end
%iterate to improve the values
D=mp(diag(D));V=mp(V);tol=eps;     
for i=1:length(D)
    SS.type='()';SS.subs=builtin('cell',{i});sigma=subsref(D,SS);oldGuess=2*sigma;
    SS.type='()';SS.subs=builtin('cell',{':' i});guessV=subsref(V,SS);
    %original guessD=2*D(i);guessV=V(:,i);sigma=D(i);
    if isreal(sigma) %we can deal only with real eigenvalues; the complex ones will be left as class double
        %we will perform inverse iteration with shift
        AA=inv(A-sigma*eye(size(A)));%When we have a better LU decomposition, we can get rid of "inv"
        iter=0;
        while abs((oldGuess-sigma)/(oldGuess+sigma))>tol & iter<100
            t=AA*guessV;oldGuess=sigma;
            sigma=guessV'*t;
            guessV=t/norm(t);
            iter=iter+1;
        end
        %Recover the original situation, by using the already improved eigenvector to
        %evaluate the corresponding eigenvalue
        %original D(i)=sigma;V(:,i)=guessV;
        sigma=guessV'*A*guessV;
        SS.type='()';SS.subs=builtin('cell',{i});D=subsasgn(D,SS,sigma);
        SS.type='()';SS.subs=builtin('cell',{':' i});V=subsasgn(V,SS,guessV);
    end
end
D=diag(D);
